/*

Copyright (C) 2000, 2001, 2002 Christian Kreibich <christian@whoop.org>.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
#ifndef __nd_gui_h
#define __nd_gui_h

#include <gtk/gtk.h>
#include <netdude/nd_packet.h>
#include <netdude/nd_trace.h>

typedef void (*ND_MenuEntryCB) (ND_Packet   *packet,
				guchar      *header,
				int          value);

struct nd_menu_data
{
  const char                   *label;
  const char                   *tooltip;
  int                           value;
  ND_MenuEntryCB                callback;
};


typedef enum {
  ND_COLOR_BG,
  ND_COLOR_RED,
  ND_COLOR_YELLOW
} ND_GuiColor;

/* Initializes GUI stuff -- progress bar, tooltips etc. */
void         nd_gui_init(void);

/* This function sets the various GUI elements sensitive/
   insensitive according to the current packet/trace selection. */
void         nd_gui_sync(void);

/* This function pops up the packet menu when the user
   right-clicks a line in the packet list. */
void         nd_gui_show_packet_menu(GdkEventButton *event);

void         nd_gtk_clist_select_all(GtkCList *clist);
void         nd_gtk_clist_unselect_all(GtkCList *clist);

/* Updates the field in the lower left containing
   the number of packets of the current trace. */
void         nd_gui_num_packets_set(void);

void         nd_gui_modified_set(ND_Trace *trace);

/* Sets the status bar content to a given message. */
void         nd_gui_statusbar_set(const char *text);
gint         nd_gui_statusbar_clear(gpointer data);

/* Initializes/resets the progress bar to a given
   number of steps between 0 and 100%. */
void         nd_gui_pbar_reset(int num_steps);
void         nd_gui_pbar_inc(void);
void         nd_gui_pbar_clear(void);

/* Starts activity display in the progress bar
   (a small bar that slides back and forth */
void         nd_gui_pbar_start_activity(void);
/* Stops activity display. */
void         nd_gui_pbar_stop_activity(void);

/* Enables/disables display of the column in the
   clist that indicates whether a packet was snarfed
   completely. */
void         nd_gui_list_incomplete_column_visible(GtkWidget *clist, gboolean visible);
/* Toggles a given line's completeness display. */
void         nd_gui_list_incomplete_row_set(GtkWidget *clist, int row, gboolean incomplete);
void         nd_gui_list_remove_selected_rows(GtkWidget *clist);
void         nd_gui_list_remove_row(GtkWidget *clist, guint index);

void         nd_gui_list_update(ND_Trace *trace, gboolean find_context);
void         nd_gui_list_update_packet(const ND_Packet *p);
void         nd_gui_list_update_packet_at_index(const ND_Packet *p, int index);
void         nd_gui_list_update_packet_state(const ND_Packet *p);
void         nd_gui_list_update_packet_state_at_index(const ND_Packet *p, int index);

void         nd_gui_windowtitle_set(const char *filename);

/* Requests the timestamp tooltip to be displayed in a bit. */
void         nd_gui_timestamp_schedule(GtkCList *clist, GdkEventMotion *event);
/* Hides the timestamp tooltip. */
void         nd_gui_timestamp_hide(void);

/* Protocol implementations should use this one to hook
   their menus in the menu's pulldown menu . */
void         nd_gui_proto_menu_register(ND_Protocol *proto);

/* Adds a trace to the GUI */
void         nd_gui_trace_add(ND_Trace *trace);
/* Removes a trace from the GUI */
void         nd_gui_trace_remove(ND_Trace *trace);

/* Creates the GUI parts that a trace needs (the contents of
   its tab in the mainwindow notebook) and hooks them into
   the given trace. */
void         nd_gui_trace_new_tab(ND_Trace *trace);

void         nd_gui_trace_name_set(ND_Trace *trace);


void         nd_gui_widget_set_color(GtkWidget *widget, ND_GuiColor color);


/* Creates a menu out of the given ND_MenuData array and
   returns it. The last entry in the array must have a
   label value of NULL. */
GtkWidget   *nd_gui_create_menu(ND_MenuData *data);

/* Creates a table widget based upon info contained in the fields
   array. The last array entry MUST contain NULL values.
   Each field's widget can be obtained from the returned
   widget using gtk_object_get_data with the field's label
   as a key. The returned widgets are GtkButtons. */
GtkWidget   *nd_gui_proto_table_create(ND_Trace *trace,
				       ND_ProtoInfo *pinf);

/* Reset varying fields for given protocol. Must be called
 * before any nd_gui_proto_table_add() calls! */
void         nd_gui_proto_table_clear(ND_Trace *trace,
				      ND_ProtoInfo *pinf);

void         nd_gui_proto_table_add(ND_Trace *trace,
				    ND_ProtoInfo *pinf,
				    ND_ProtoField *field,
				    void *data,
				    gboolean is_error);


void         nd_gui_proto_table_block_events(ND_Trace *trace, ND_ProtoInfo *pi);
void         nd_gui_proto_table_unblock_events(ND_Trace *trace, ND_ProtoInfo *pi);

void         nd_gui_add_monowidth_widget(GtkWidget *widget);
void         nd_gui_del_monowidth_widget(GtkWidget *widget);
void         nd_gui_update_monowidth_widgets(void);

#endif
